#!/bin/sh
#                                       
# Start the L2TP server daemon
#                                                                               

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNCenter"
LOCALIP=`/bin/get_key_value ${PKG_USERCONF_DIR}/l2tp/xl2tpd.conf "local ip"`
PKG_TARGET="/var/packages/VPNCenter/target"
KERNEL_MODULES="slhc.ko ppp_generic.ko \
		ppp_async.ko arc4.ko ppp_mppe.ko bsd_comp.ko zlib_deflate.ko ppp_deflate.ko \
		x_tables.ko ip_tables.ko iptable_filter.ko nf_conntrack.ko nf_defrag_ipv4.ko nf_conntrack_ipv4.ko \
		nf_nat.ko iptable_nat.ko ipt_REDIRECT.ko xt_multiport.ko xt_tcpudp.ko xt_state.ko ipt_MASQUERADE.ko"
IPSec_MODULES="hmac.ko xfrm_user.ko af_key.ko xfrm_ipcomp.ko ah4.ko ah6.ko esp4.ko esp6.ko tunnel4.ko tunnel6.ko xfrm4_tunnel.ko xfrm6_tunnel.ko ipcomp.ko ipcomp6.ko authenc.ko authencesn.ko deflate.ko xfrm4_mode_beet.ko xfrm6_mode_beet.ko xfrm4_mode_tunnel.ko xfrm6_mode_tunnel.ko xfrm4_mode_transport.ko xfrm6_mode_transport.ko"
SERVICE="l2tp"
BIN_IPTABLESTOOL="/usr/syno/bin/iptablestool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"
if [ -x "/usr/sbin/ipsec" ]; then
	SBIN_IPSEC="/usr/sbin/ipsec"
elif [ -x "/usr/local/sbin/ipsec" ]; then
	SBIN_IPSEC="/usr/local/sbin/ipsec"
else
	exit 1
fi

# where the private directory and the config files are
IPSEC_CONFS="${PKG_USERCONF_DIR}/l2tp"
export IPSEC_CONFS

reverse_modules() {
	local modules=$1
	local mod
	local ret=""

	for mod in $modules; do
	    ret="$mod $ret"
	done

	echo $ret
}

IPSec_Mod=""
for mod in $KERNEL_MODULES; do
	if [ -e "/lib/modules/$mod" ]; then
		IPSec_Mod="${IPSec_Mod} ${mod}"
	fi
done
for mod in $IPSec_MODULES; do
	if [ -e "/lib/modules/$mod" ]; then
		IPSec_Mod="${IPSec_Mod} ${mod}"
	fi
done

start() { 
	Run=`/bin/get_key_value ${PKG_USERCONF_DIR}/synovpn.conf runl2tpd`
	if [ "yes" != "$Run" ]; then
		exit 0;
	fi
	        
	echo "Starting L2TP server: "
	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --insmod $SERVICE ${IPSec_Mod}
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		/usr/syno/bin/iptablestool --insmod $SERVICE ${IPSec_Mod}
	fi

	if [ -n "`pidof xl2tpd`" ]; then
	   echo "Already running"
	   return
	fi                                                                          
	if [ ! -d "/var/run/xl2tpd" ]; then
		/bin/mkdir -p /var/run/xl2tpd
	fi
	/usr/sbin/xl2tpd -c ${PKG_USERCONF_DIR}/l2tp/xl2tpd.conf -p /var/run/xl2tpd.pid
	echo 1 > /proc/sys/net/ipv4/ip_forward
        
	/sbin/iptables -t nat -A POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE
	for f in /proc/sys/net/ipv4/conf/*;
	do
		echo 0 > $f/send_redirects; 
		echo 0 > $f/accept_redirects;
	done
	${SBIN_IPSEC} addconn --checkconfig --verbose
	if [ $? != 0 ]; then
		exit 1
	fi

	#clear pluto dir to avoid unexpected shutdown
	/bin/rm -f /var/run/pluto/*

	${SBIN_IPSEC} setup start
	${PKG_TARGET}/scripts/radiusd.sh start
}                                                                               

stop() {
	local modules=`reverse_modules "${IPSec_Mod}"`
	echo -n "Shutting down L2TP server: "

	#disconnect clients
	if [ -n "${LOCALIP}" ]; then
		/bin/kill `ps w | /bin/grep pppd | /bin/grep l2tp | /bin/grep "${LOCALIP}" | awk '{print $1;}'` 2>/dev/null
	fi
	
        /sbin/iptables -t nat -D POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE
	
	/bin/kill `cat /var/run/xl2tpd.pid` 2>/dev/null
	${SBIN_IPSEC} setup stop
	sleep 1
	if [ -n "`pidof xl2tpd`" ]; then
	    echo "Failed to stop xl2tpd"
	fi

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE $modules
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		/usr/syno/bin/iptablestool --rmmod $SERVICE $modules
	fi

	${PKG_TARGET}/scripts/radiusd.sh stop
}                                                                              
                                                                               
case "$1" in
	start)
		start
	;;
	stop)
		stop
	;;
	restart|reload)
		stop
		start
	;;
	*)
	echo "Usage: $0 {start|stop|restart}"
	exit 1
esac
exit $?
