#!/bin/sh
#
# Startup script for openvpn server
#

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNCenter"
CONF_DIR="${PKG_USERCONF_DIR}/openvpn"
USER_CONF="openvpn.conf.user"
USER_CONF_TMP_FILE="/tmp/run.openvpn.conf.user"
OPENVPN_CONF="openvpn.conf"
PKG_TARGET="/var/packages/VPNCenter/target"
KERNEL_MODULES="x_tables.ko ip_tables.ko iptable_filter.ko nf_conntrack.ko nf_defrag_ipv4.ko nf_conntrack_ipv4.ko nf_nat.ko iptable_nat.ko ipt_REDIRECT.ko xt_multiport.ko xt_tcpudp.ko xt_state.ko ipt_MASQUERADE.ko tun.ko"
SERVICE="openvpn"
BIN_IPTABLESTOOL="/usr/syno/bin/iptablestool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"

reverse_modules() {
	local modules=$1
	local mod
	local ret=""

	for mod in $modules; do
	    ret="$mod $ret"
	done

	echo $ret
}

if [ -e ${CONF_DIR}/${USER_CONF} ]; then
	OPENVPN_CONF=${USER_CONF}
fi

SYNOVPN_CONF="${PKG_USERCONF_DIR}/synovpn.conf"
VPNC_CURRENT="/tmp/vpnc_current"
LOCALIP=`grep "^server " ${CONF_DIR}/${OPENVPN_CONF} | cut -d' ' -f 2`

case "$1" in
  start)
        local Run=`/bin/get_key_value ${SYNOVPN_CONF} runopenvpn`
	if [ "yes" != "$Run" ]; then
		exit 0;
	fi

	local RunningProto=`/bin/get_key_value ${VPNC_CURRENT} proto`
	if [ "openvpn" = "$RunningProto" ]; then
		echo "Already running"
		exit 0;
	fi

	echo 1 > /proc/sys/net/ipv4/ip_forward

	# Make device if not present (not devfs)
	if ( [ ! -c /dev/net/tun ] ) then
  		# Make /dev/net directory if needed
  		if ( [ ! -d /dev/net ] ) then
        		mkdir -m 755 /dev/net
  		fi
  		mknod /dev/net/tun c 10 200
	fi

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --insmod $SERVICE ${KERNEL_MODULES}
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		/usr/syno/bin/iptablestool --insmod $SERVICE ${KERNEL_MODULES}
	else
		for i in x_tables.ko ip_tables.ko iptable_filter.ko nf_conntrack.ko \
			nf_defrag_ipv4.ko nf_conntrack_ipv4.ko nf_nat.ko \
			iptable_nat.ko ipt_REDIRECT.ko xt_multiport.ko xt_tcpudp.ko \
			xt_state.ko ipt_MASQUERADE.ko tun.ko
		do
			/sbin/insmod /lib/modules/$i
		done
	fi

        # create links : ca.crt, ca_bundle.crt, server.crt, and server.key of ssl
        if [ ! -e ${PKG_TARGET}/etc/openvpn/keys/ca.crt ]; then
                ln -sf ${PKG_USERCONF_DIR}/openvpn/keys/ca.crt ${PKG_TARGET}/etc/openvpn/keys/
                ln -sf ${PKG_USERCONF_DIR}/openvpn/keys/ca_bundle.crt ${PKG_TARGET}/etc/openvpn/keys/
                ln -sf ${PKG_USERCONF_DIR}/openvpn/keys/server.crt ${PKG_TARGET}/etc/openvpn/keys/
                ln -sf ${PKG_USERCONF_DIR}/openvpn/keys/server.key ${PKG_TARGET}/etc/openvpn/keys/
        fi

	# start radius server
	${PKG_TARGET}/scripts/radiusd.sh start

        echo "Starting openvpn ..."
	${PKG_TARGET}/sbin/openvpn --daemon --cd ${CONF_DIR} --config ${OPENVPN_CONF} --writepid /var/run/ovpn_server.pid

	/sbin/iptables -t nat -A POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE

	#touch a temp file for UI check
	if [ ${OPENVPN_CONF} == ${USER_CONF} ]; then
		/bin/touch ${USER_CONF_TMP_FILE} 2>/dev/null
	elif [ -e ${USER_CONF_TMP_FILE} ]; then
		/bin/rm -rf ${USER_CONF_TMP_FILE} 2>/dev/null
	fi
        
        ;;
  stop)
	local Run=`/bin/get_key_value ${SYNOVPN_CONF} runopenvpn`
	if [ "yes" != "$Run" ]; then
		exit 0;
	fi

	local modules=`reverse_modules "${KERNEL_MODULES}"`
        /sbin/iptables -t nat -D POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE

        echo "Stopping openvpn ..."
	OPENVPN_PID=`cat /var/run/ovpn_server.pid`
	/bin/kill ${OPENVPN_PID} 2>/dev/null
	sleep 1

	kill -0 ${OPENVPN_PID} > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		echo "Failed to stop openvpn server"
	fi

	sleep 2	
	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE $modules
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		/usr/syno/bin/iptablestool --rmmod $SERVICE $modules
	else
		/sbin/rmmod tun

		/sbin/rmmod ipt_MASQUERADE xt_state xt_tcpudp xt_multiport \
			ipt_REDIRECT iptable_nat nf_nat nf_conntrack_ipv4 \
			nf_defrag_ipv4 nf_conntrack iptable_filter ip_tables x_tables
	fi

	${PKG_TARGET}/scripts/radiusd.sh stop

	#delete USER_CONF_TMP_FILE
	if [ -e ${USER_CONF_TMP_FILE} ]; then
		/bin/rm -rf ${USER_CONF_TMP_FILE} 2>/dev/null
	fi

	;;
  restart)
        sh $0 stop quiet
        sleep 3
        sh $0 start
        ;;
  *)
        echo "Usage: $0 {start|stop|restart}"
        exit 1
esac

exit 0

# [EOF]



