#!/bin/sh
#                                       
# Start the PPTP server daemon
#                                                                               

PKG_USERCONF_DIR="/usr/syno/etc/packages/VPNCenter"
LOCALIP=`grep "^localip" ${PKG_USERCONF_DIR}/pptp/pptpd.conf | cut -d' ' -f 2`
BCRELAY=`grep "^bcrelay" /usr/syno/etc/synovpn/pptp/pptpd.conf | cut -d' ' -f 2`
PKG_TARGET="/var/packages/VPNCenter/target"
KERNEL_MODULES="slhc.ko ppp_generic.ko ppp_async.ko arc4.ko ppp_mppe.ko bsd_comp.ko zlib_deflate.ko ppp_deflate.ko x_tables.ko ip_tables.ko iptable_filter.ko nf_conntrack.ko nf_defrag_ipv4.ko nf_conntrack_ipv4.ko nf_nat.ko iptable_nat.ko ipt_REDIRECT.ko xt_multiport.ko xt_tcpudp.ko xt_state.ko ipt_MASQUERADE.ko"
SERVICE="pptp"
BIN_IPTABLESTOOL="/usr/syno/bin/iptablestool"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"

reverse_modules() {
	local modules=$1
	local mod
	local ret=""

	for mod in $modules; do
	    ret="$mod $ret"
	done

	echo $ret
}

PPTP_Mod=""
for mod in $KERNEL_MODULES; do
	if [ -e "/lib/modules/$mod" ]; then
		PPTP_Mod="${PPTP_Mod} ${mod}"
	fi
done

start() { 
	Run=`/bin/get_key_value ${PKG_USERCONF_DIR}/synovpn.conf runpptpd`
	if [ "yes" != "$Run" ]; then
		exit 0;
	fi
	        
	echo -n "Starting PPTP server: "
	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --insmod $SERVICE ${PPTP_Mod}
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		/usr/syno/bin/iptablestool --insmod $SERVICE ${PPTP_Mod}
        else
		for i in slhc.ko ppp_generic.ko ppp_async.ko arc4.ko ppp_mppe.ko \
			bsd_comp.ko zlib_deflate.ko ppp_deflate.ko x_tables.ko \
			ip_tables.ko iptable_filter.ko nf_conntrack.ko \
			nf_defrag_ipv4.ko nf_conntrack_ipv4.ko nf_nat.ko \
			iptable_nat.ko ipt_REDIRECT.ko xt_multiport.ko xt_tcpudp.ko \
			xt_state.ko ipt_MASQUERADE.ko
		do
			/sbin/insmod /lib/modules/$i
		done
	fi

	if [ -n "`pidof pptpd`" ]; then
	   echo "Already running"
	   return
	fi                                                                          
	${PKG_TARGET}/sbin/pptpd -c ${PKG_USERCONF_DIR}/pptp/pptpd.conf -o ${PKG_USERCONF_DIR}/pptp/options.pptpd -p /var/run/pptpd.pid
	echo 1 > /proc/sys/net/ipv4/ip_forward
        
        /sbin/iptables -t nat -A POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE
	
	${PKG_TARGET}/scripts/radiusd.sh start
}                                                                               
stop() {
	local modules=`reverse_modules "${PPTP_Mod}"`
	echo -n "Shutting down PPTP server: "

	#disconnect clients
	if [ -n "${LOCALIP}" ]; then
		/bin/kill `ps w |/bin/grep pppd | /bin/grep pptp | /bin/grep "${LOCALIP}" | awk '{print $1;}'` 2>/dev/null
	fi

        /sbin/iptables -t nat -D POSTROUTING -s ${LOCALIP}/24 -j MASQUERADE
	
	/bin/kill `cat /var/run/pptpd.pid` 2>/dev/null
	sleep 1
	if [ -n "`pidof pptpd`" ]; then
	    echo "Failed to stop pptpd"
	fi
	#echo 0 > /proc/sys/net/ipv4/ip_forward

	if [ -x ${BIN_SYNOMODULETOOL} ]; then
		$BIN_SYNOMODULETOOL --rmmod $SERVICE $modules
	elif [ -x ${BIN_IPTABLESTOOL} ]; then
		/usr/syno/bin/iptablestool --rmmod $SERVICE $modules
	else
		/sbin/rmmod ipt_MASQUERADE xt_state xt_tcpudp xt_multiport \
			ipt_REDIRECT iptable_nat nf_nat nf_conntrack_ipv4 \
			nf_defrag_ipv4 nf_conntrack iptable_filter ip_tables x_tables
		/sbin/rmmod ppp_deflate zlib_deflate bsd_comp ppp_mppe arc4 ppp_async \
			ppp_generic slhc
	fi

	${PKG_TARGET}/scripts/radiusd.sh stop
}                                                                              
                                                                               
case "$1" in
	start)
		start
	;;
	stop)
		stop
	;;
	restart|reload)
		stop
		start
	;;
	condrestart)
		if [ -r /opt/var/run/pptpd.pid ]; then
			stop
			start
		fi
	;;
	*)
	echo "Usage: $0 {start|stop|restart|condrestart}"
	exit 1
esac
exit $?
